<?php

namespace Drupal\jsonapi_include\EventSubscriber;

use Drupal\Core\Cache\CacheableResponseInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\jsonapi\Routing\Routes;
use Drupal\jsonapi_include\JsonapiParseInterface;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpKernel\Event\ResponseEvent;
use Symfony\Component\HttpKernel\KernelEvents;

/**
 * The class handler response subscriber.
 */
class ResponseSubscriber implements EventSubscriberInterface {

  public function __construct(
    protected ConfigFactoryInterface $config,
    protected JsonapiParseInterface $jsonapiInclude,
    protected RouteMatchInterface $routeMatch,
  ) {}

  /**
   * {@inheritdoc}
   */
  #[\Override]
  public static function getSubscribedEvents(): array {
    $events[KernelEvents::RESPONSE] = ['onResponse'];

    return $events;
  }

  /**
   * This method is called the KernelEvents::RESPONSE event is dispatched.
   *
   * @param \Symfony\Component\HttpKernel\Event\ResponseEvent $event
   *   The response event.
   */
  public function onResponse(ResponseEvent $event): void {

    if (!$this->routeMatch->getRouteObject()) {
      return;
    }
    $route_defaults = $this->routeMatch->getRouteObject()->getDefaults();
    if (Routes::isJsonApiRequest($route_defaults) || !empty($route_defaults[Routes::JSON_API_ROUTE_FLAG_KEY])) {
      $response = $event->getResponse();
      if ($response instanceof CacheableResponseInterface) {
        $response->getCacheableMetadata()->addCacheContexts(['url.query_args:jsonapi_include']);
      }
      $need_parse = TRUE;
      if ($this->config->get('jsonapi_include.settings')->get('use_include_query')) {
        $need_parse = !empty($event->getRequest()->query->get('jsonapi_include'));
      }

      if ($need_parse) {
        $content = $response->getContent();
        if (str_starts_with($content, '{"jsonapi"')) {
          $this->jsonapiInclude->parse($response);
        }
      }
    }
  }

}
